<?php
define("_PAYPAL_SITE_", "ssl://www.paypal.com");
define("_PAYPAL_SITE_TEST", "ssl://www.sandbox.paypal.com");

$path_extra = dirname(__FILE__) . "/lib";

$path = ini_get('include_path');
$path = $path_extra . PATH_SEPARATOR . $path;
ini_set('include_path', $path);

@include_once('template/configdir.php');
@include_once(_DATA_DIR_ . 'initdb.php');
@include_once(_DATA_DIR_ . 'set.php');

include_once('init.php');
require_once("dao.php");
require_once("Rt.php");
require_once 'mail.php';

$templateDir = dirname(__FILE__) . '/template/';

/**
 * 支払い完了処理
 *
 * @return bool
 */
function checkCompleted(){
  global $dao;
  $item_name = isset($_POST['item_name']) ? $_POST['item_name'] : "";
	$item_number = isset($_POST['item_number']) ? $_POST['item_number'] : "";
	$txn_id = isset($_POST['txn_id']) ? $_POST['txn_id'] : "";
	$receiver_email = isset($_POST['receiver_email']) ? $_POST['receiver_email'] : "";
	$payer_email = isset($_POST['payer_email']) ? $_POST['payer_email'] : "";
	$payment_date = isset($_POST['payment_date']) ? $_POST['payment_date'] : "";
	$payer_id = isset($_POST['payer_id']) ? $_POST['payer_id'] : "";
	$receiver_id = isset($_POST['receiver_id']) ? $_POST['receiver_id'] : "";
	$mc_gross = isset($_POST['mc_gross']) ? $_POST['mc_gross'] : "";
	$mc_currency = isset($_POST['mc_currency']) ? $_POST['mc_currency'] : "";

	if($txn_id == ""){
		$dao->insertErrLog("取引処理:取引IDが無い");
		return false;
	}

	// 取引の重複の排除
	$ret = $dao->isExistTrade($txn_id);
	if($ret){	// 既に処理済み
		$dao->insertErrLog("取引処理:取引記録の重複:" . $txn_id);
		return false;
	}
		
	// 商品のチェック
	if($item_number == ""){
		$dao->insertErrLog("取引処理：商品ID指定なし：" . $txn_id);
		return false;
	}

	$resultGoods = $dao->getItemById($item_number);
	if(count($resultGoods) == 0){
        $dao->insertErrLog("取引処理：商品なし：" . $txn_id);
		return false;
	}

/*    
	// テキストデータかつ、小さな文章であれば、メールに添付する
	mb_language("ja");
	mb_internal_encoding("UTF-8");
	$goodsData = "";
	$contentType = $resultGoods['contenttype'];
	$contentSize = $resultGoods['contentsize'];    
	if((strcmp($contentType, "text/plain") == 0) && ($contentSize < 500)){
		$resultGoods2 = $dao->getItemDataById($receiver_email, $item_number);
		if(count($resultGoods2) == 0){
			$dao->insertLog("取引処理:商品情報なし2 " . $item_number);
		}else{
			$goodsData = $resultGoods2['contentdata'];
			$encodeType = mb_detect_encoding($goodsData);
			if(strcmp($encodeType, "UTF-8") != 0){
				$goodsData = mb_convert_encoding($goodsData, "UTF-8", "auto");
			}
		}
	}
*/
	// check that receiver_email is your Primary PayPal email
	if(strcmp($receiver_email, _OWNER_EMAIL_) != 0){
		$dao->insertErrLog("取引処理:受け取りユーザーが異なる " . $receiver_email . " " . _OWNER_EMAIL_);
		return false;
	}
	
	// 金額のチェック
	$price = $resultGoods['price'];
	if($price != $mc_gross){
		$dao->insertErrLog("取引処理:金額の不一致 " . $price . ":" . $mc_gross);
		return false;	
	}
	if(strcmp($mc_currency,"JPY") != 0){
		$dao->insertErrLog("取引処理:通貨の不一致 " . $mc_currency);
		return false;
	}

	// 取引を記録する
	$dao->insertTrade($txn_id, $item_number, $receiver_id, $receiver_email, $payer_id, $payer_email, $payment_date, $mc_gross, $mc_currency);

	// ダウンロード許可を与える
	$dao->insertPermitDL($txn_id, $item_number, $payer_id, $payer_email, _DEFAULT_TERM_);

	$url = _SITE_URL_ . "download.php?txn_id="
		. urlencode(stripslashes($txn_id))
		. "&payer_email=" . urlencode(stripslashes($payer_email));

	// メールの送信 
/*
    if($goodsData == ""){
		$note = "";
	}else{
		$note = getResourceText("TextData", $goodsData);
	} 
 */
	$limitDate = time() + (3600 * 24 * _DEFAULT_TERM_);

	$msgMail = getMailTextPayer($item_name, $item_number, $mc_gross, $mc_currency, $receiver_email, $url, $limitDate);					
	mb_send_mail($payer_email, "ダウンロードのお知らせ",$msgMail, "From: " . _OWNER_EMAIL_);		

	// 受け取り側
	if(true){
		$msgMailRec = getMailTextReciver($txn_id, $item_name, $item_number, $mc_gross, $mc_currency, $payer_email, $limitDate);
		mb_send_mail($receiver_email, "販売のお知らせ",$msgMailRec, "From: " . _OWNER_EMAIL_);
	}
	
	return true;
}

/**
 * キャンセル処理
 *
 * @return unknown
 */
function checkRefunded(){
  global $dao;
  $item_name = isset($_POST['item_name']) ? $_POST['item_name'] : "";
	$item_number = isset($_POST['item_number']) ? $_POST['item_number'] : "";
	$mc_gross = isset($_POST['mc_gross']) ? $_POST['mc_gross'] : "";
	$mc_currency = isset($_POST['mc_currency']) ? $_POST['mc_currency'] : "";
	$txn_id = isset($_POST['txn_id']) ? $_POST['txn_id'] : "";
	$parent_txn_id = isset($_POST['parent_txn_id']) ? $_POST['parent_txn_id'] : "";
	$receiver_email = isset($_POST['receiver_email']) ? $_POST['receiver_email'] : "";
	$payer_email = isset($_POST['payer_email']) ? $_POST['payer_email'] : "";
	$payment_date = isset($_POST['payment_date']) ? $_POST['payment_date'] : "";
	$payer_id = isset($_POST['payer_id']) ? $_POST['payer_id'] : "";
	$receiver_id = isset($_POST['receiver_id']) ? $_POST['receiver_id'] : "";
	
	if($txn_id == ""){
		$dao->insertErrLog("返金処理:取引IDが無い");
		return false;
	}
	if($parent_txn_id == ""){
		$dao->insertErrLog("返金処理:返金対象の取引IDが無い");
		return false;
	}
	// 取引の重複の排除
	if($dao->isExistRefunded($txn_id)){	// 既に処理済み
		$dao->insertErrLog("返金処理:返金取引記録の重複:" . $txn_id);
		return false;
	}
	
	if($dao->isExistTrade($parent_txn_id) == false){
		$dao->insertErrLog("返金処理:取引IDが未登録 " . $parent_txn_id);
		return false;
	}
	
	// 商品のチェック
	if($item_number == ""){
		$dao->insertErrLog("返金処理:商品番号がない");
		return false;
	}
	// 商品のチェック　返金処理のため、金額が一致しなかったり、商品が既に無い可能性がある
	// 返金処理までの期間に、メールアドレスが変更されると、ここで、はじいてしまう。IDを比較すれば解決するが、IDを使ってよいのか不明
	$flg = true;
	$resultGoods = $dao->getItemById($item_number);
	if(is_null($resultGoods)){
		$flg = false;
		$dao->insertErrLog("返金処理:商品情報なし " . $item_number);
	}else {
		// 金額のチェック
		$price = $resultGoods['price'];
		if($price != ($mc_gross * -1)){
			$flg = false;
//			$dao->insertErrLog("返金処理:金額の不一致 " . $price . ":" . $mc_gross);
		}else if(strcmp($mc_currency,"JPY") != 0){
			$flg = false;
//			$dao->insertErrLog("返金処理:通貨の不一致 " . $mc_currency);
		}
	}

	// check that receiver_email is your Primary PayPal email
//	if(strcmp($receiver_email, $ownerMail) != 0){
//		$dao->insertErrLog("返金処理:受け取りユーザーが異なる " . $receiver_email . " " . $ownerMail);
//		return false;
//	}
	
	// 取引を記録する
	$dao->insertRefunded($txn_id, $parent_txn_id, $receiver_id, $receiver_email, $payer_id, $payer_email, $payment_date, $mc_gross, $mc_currency);
	
	// ダウンロード許可を削除する
	$dao->deletePermitDL($parent_txn_id);
	
	// メールの送信
	mb_language("ja");
	mb_internal_encoding("UTF-8");

	$msgMail = getMailTextRefundedPayer($item_name, $item_number, $mc_gross, $mc_currency, $receiver_email);
					
	mb_send_mail($payer_email, "ダウンロード停止のお知らせ",$msgMail, "From: " . _OWNER_EMAIL_);		

	// 受け取り側
	if(true){
		$msgMail2 = getMailTextRefundedReciver($item_name, $item_number, $mc_gross, $mc_currency, $payer_email);
		mb_send_mail($receiver_email, "ダウンロード停止のお知らせ",$msgMail2, "From: " . _OWNER_EMAIL_);
	}
	
	return true;
}

$dao = new Dao();
if(isset($_SERVER['REMOTE_ADDR'])){
  $remodeAddr = $_SERVER['REMOTE_ADDR'];
  $hostname = gethostbyaddr($remodeAddr);
}else{
  $remodeAddr = "unknown";
  $hostname = "";
}
$dao->insertLog("accesslog:" . $remodeAddr . ":" . $hostname);

// 不正アクセスの防止
if((strcmp($hostname, "ipn.paypal.com") != 0) && (strcmp($hostname, "notify.paypal.com") != 0)
	&& (strcmp($hostname, "ipn.sandbox.paypal.com") != 0))
{
	exit();
}

$req = 'cmd=_notify-validate';

foreach ($_POST as $key => $value) {
	$value = urlencode(stripslashes($value));
	$req .= "&$key=$value";
}

$flgTest = false;
if(isset($_POST["test_ipn"])){
	if($_POST["test_ipn"] == "1"){
		$flgTest = true;
	}
}

if(defined("_SANDBOX_")){
	if(!$flgTest){
		$dao->insertErrLog("SANDBOX使用中に、正式なアクセスを受けた");
		exit();
	}
	$payPalUrl = _PAYPAL_SITE_TEST;
}else{
	if($flgTest){
		$dao->insertErrLog("正式運用中に、SANDBOXからのアクセスを受けた");
		exit();
	}
	$payPalUrl = _PAYPAL_SITE_;	
}

$dao->insertLog(print_r($_POST, true));

// post back to PayPal system to validate
$header .= "POST /cgi-bin/webscr HTTP/1.1\r\n";
$header .= "Content-Type: application/x-www-form-urlencoded\r\n";
//$header .= "Content-Length: " . strlen($req) . "\r\n\r\n";
$header .= "Host: www.paypal.com\r\n";
$header .= "Content-Length: " . strlen($req) . "\r\n";
$header .= "Connection: close\r\n\r\n";

//$fp = fsockopen ($payPalUrl, 80, $errno, $errstr, 30);
$fp = fsockopen ($payPalUrl, 443, $errno, $errstr, 30);

// assign posted variables to local variables
$payment_status = isset($_POST['payment_status']) ? $_POST['payment_status'] : "";

$resData = "";
if (!$fp) {
	// HTTP ERROR
} else {
	fputs ($fp, $header . $req);
	while (!feof($fp)) {
		$res = fgets ($fp, 1024);
        $resData .= $res;
		if (strcmp ($res, "VERIFIED") == 0) {
			// check the payment_status is Completed
			if(strcmp($payment_status, "Completed") == 0){
				checkCompleted();
			}else if(strcmp($payment_status, "Refunded") == 0){
				checkRefunded();
			}
		}
		else if (strcmp ($res, "INVALID") == 0) {
			// log for manual investigation
			$dao->insertLog("INVALID");
		}
	}
	fclose ($fp);
}

//$dao->insertLog($resData);
?>