<?php
/**
 * HtmlのSelect部分を作成するクラス
 *
 * 使い方
 * $htmlSelect = new HtmlSelect();
 * $htmlSelect->setName("local");
 * $htmlSelect->setOption(array("a"=>"東京","b"=>"大阪"));
 * $str = $htmlSelect->getHtml();
 */
class HtmlSelect {
  protected $isMulti; // 複数選択可能

  /*
   * select部分に与える値の配列
   *    $s['name'] = "local";
   *    $s['id'] = "local";
   * $option オプションに与える値の配列
   *    $opt[0]['value'] = "0";
   *    $opt[0]['text'] = "東京";
   *    $opt[0]['selected'] = "selected";
   *    $opt[1]['value'] = "";
   *    $opt[1]['text'] = "大阪";
   */
  protected $select;
  protected $option;
  
  function  __construct($isMulti = false) {
    $this->isMulti = $isMulti;
    if($isMulti){
      $select['multiple'] = "true";
    }
    $this->select = array();
    $this->option = array();
    
  }

  /**
   * name属性
   * @param string $name
   */
  function setName($name) {
    $this->select['name'] = $name;
  }

  /**
   * select部分の他の属性
   * @param string $key
   * @param string $value 
   */
  function setSelectAtrb($key, $value){
    $this->select[$key] = $value;
  }

  /**
   * オプション
   * @param array $option オプションの配列 例 array("a"=>"東京","b"=>"大阪")
   * @param string $selectedKey 選択済みのキー
   */
  function setOption($option, $selectedKey=null){
    $i = 0;
    foreach($option as $key => $value){
      $this->option[$i]['value'] = $key;
      $this->option[$i]['text'] = $value;
      if(strcmp($selectedKey, $key) == 0){
        $this->option[$i]['selected'] = "selected";
      }
      $i++;
    }
  }

  /**
   * 複数選択時などに使う
   * 未確認
   * @param array $opt 配列 $ar[][属性名]=値 
   */
  function setOptionArray($opt){
    $this->option = $opt;
  }

  /**
   * htmlのselectを生成
   */
  function getHtml() {
    $select = $this->select;
    $option = $this->option;
    $data = "<select";
    foreach($select as $key => $value) {
      $data .= " " . $key . "='" . $value . "'";
    }
    $data .= ">";

    foreach($option as $opt) {
      $data .= "<option";
      $text = "";
      foreach($opt as $key => $value) {
        if(strcmp("text", $key) == 0) {
          $text = $value;
        }else {
          $data .= " " . $key . "='" . $value . "'";
        }
      }
      $data .= ">" . $text . "</option>";
    }

    $data .= "</select>";
    return $data;
  }

}

?>
