<?php
class Dao{
  	protected $dbName = _DB_NAME_;	
	protected $userName = _DB_USER_;
	protected $userPassword = _DB_PASSWORD_;
	private $dbh;
    
	function insertItem($item_number, $item_name, $filename, $filenameLocal, $contentText, $contenttype, $contentsize, $price, $currency){
      $regDate = time();	
      $sql = "insert into goods (item_number, item_name, filename, filenamelocal, contenttext, contenttype, contentsize, price, mc_currency,downloadcnt, regdate,editdate) values(?,?,?,?,?,?,?,?,?,?,?,?)";
      $this->insert($sql, $item_number, $item_name, $filename, $filenameLocal, $contentText, $contenttype, $contentsize, $price, $currency, 3,$regDate,$regDate);
	}    

	/**
	 * 指定の商品の情報を得る。
	 *
	 * @param string $item_number
	 * @return array
	 */
	function getItemById($item_number){
		$sql = "select item_number, item_name, filename, filenamelocal, contenttext, contenttype, contentsize, price, mc_currency,downloadcnt, regdate,editdate from goods where item_number=?";
		$ret = $this->get($sql, $item_number);
		return $ret;
	}

	/**
	 * 商品情報の取得
	 *
	 * @param int $offset
	 * @param int $num
	 * @return array
	 */
	function selectItem($offset, $num){
		$sql = "select item_number, item_name, filename, filenamelocal, contenttext, contenttype, contentsize, price, mc_currency,downloadcnt, regdate,editdate from goods limit " . $offset . "," . $num;
		$rows = $this->select($sql);
		return $rows;
	}

  function deleteItem($goodsId) {
    $sql = "delete from goods where item_number=?";
    $this->delete($sql, $goodsId);
  }    

  function updateItemInfo($orgGoodsId, $item_number, $item_name, $price, $currency) {
    $editDate = time();
    $sql = "update goods set item_number=?,item_name=?,price=?,mc_currency=?,editdate=? where item_number=?";
    $this->update($sql, $item_number, $item_name, $price, $currency, $editDate, $orgGoodsId);
  }

  function updateItemFile($goodsId, $filename, $filenameLocal, $contenttype, $contentsize) {
    $editDate = time();
    $sql = "update goods set filename=?,filenamelocal=?,contenttype=?,contentsize=?,editdate=? where item_number=?";
    $this->update($sql, $filename, $filenameLocal, $contenttype, $contentsize, $editDate,$goodsId);
  }

	/**
     * 返金処理を記録する
     * @param type $txn_id
     * @param type $parent_txn_id
     * @param type $receiver_id
     * @param type $receiver_email
     * @param type $payer_id
     * @param type $payer_email
     * @param type $payment_date
     * @param type $mc_gross
     * @param type $mc_currency
     * @return type
     */
	function insertRefunded($txn_id, $parent_txn_id, $receiver_id, $receiver_email, 
		$payer_id, $payer_email, $payment_date, $mc_gross, $mc_currency)
	{
		$sql = "insert into refunded(txn_id, parent_txn_id, receiver_id, receiver_email, payer_id, payer_email, payment_date,mc_gross,mc_currency) values(?,?,?,?,?,?,?,?,?)";
		$result = $this->insert($sql, $txn_id, $parent_txn_id, $receiver_id, $receiver_email,
			$payer_id, $payer_email, $payment_date,$mc_gross, $mc_currency);
		return $result;
	}
	
	/**
	 * 返金取引が記録されているか
	 *
	 * @param string $txn_id
	 * @return bool
	 */
	function isExistRefunded($txn_id){
		$sql = "select txn_id from refunded where txn_id=?";
		$ret = $this->isExist($sql, $txn_id);
		return $ret;
	}

	function deleteRefundedOld($term){
		$sql = "delete from refunded where regdate<subdate(curdate(), interval ? month)";
		$result = $this->delete($sql, $term);
		return $result;
	}
    
	function insertPermitDL($txn_id, $item_number, $payer_id, $payer_email, $term)
	{
		$d1 = time();
		$d2 = $d1 + ($term * 60 * 60 * 24);
		$sql = "insert into permitdl(txn_id,item_number,payer_id,payer_email,tradedate, limitdate,downloadcnt) values(?,?,?,?,?,?,0)";
		$result = $this->insert($sql, $txn_id, $item_number, $payer_id, $payer_email, $d1, $d2);
		return $result;
	}

	/**
	 * ダウンロード許可を延長する
	 *
	 * @param string $txn_id
	 * @param string $receiver_email
	 * @param unknown_type $term
	 * @return unknown
	 */
	function updatePermitDL($txn_id, $term){
		$addDate = $term * 60 * 60 * 24;
		$sql = "update permitdl set limitdate=(limitdate + ?) where (txn_id=?)";
		$this->update($sql, $addDate, $txn_id);
	}

    /**
     * ダウロードを停止する（期限を最短にする）
     * @param type $txn_id
     */
	function updatePermitDLStop($txn_id){
		$sql = "update permitdl set limitdate=tradedate where (txn_id=?)";
		$this->update($sql, $txn_id);
	}

    /**
     * ダウンロード回数を０にする
     * @param type $txn_id
     * @return type
     */
	function updatePermitDLResetCnt($txn_id){
		$sql = "update permitdl set downloadcnt=0 where (txn_id=?)";
		$result = $this->update($sql, $txn_id);
		return $result;
	}    
    
    /**
     * ダウンロード回数を増やす
     * @param type $txn_id
     * @return type
     */
	function updatePermitDLCnt($txn_id){
		$sql = "update permitdl set downloadcnt=(downloadcnt + 1) where (txn_id=?)";
		$result = $this->update($sql, $txn_id);
		return $result;
	}    
	
	/**
	 * ダウンロード許可を削除する
	 *
	 * @param string $txn_id
	 * @return unknown
	 */
	function deletePermitDL($txn_id){
		$sql = "delete from permitdl where txn_id=?";
		$result = $this->delete($sql, $txn_id);
		return $result;
	}

	function deletePermitDLByItem($item_number){
		$sql = "delete from permitdl where item_number=?";
		$result = $this->delete($sql, $item_number);
		return $result;
	}
    
	/**
	 * 期限の切れたダウンロード許可を削除する
	 *
	 * @return unknown
	 */
	function deletePermitDLOld(){
		$limitDate = time();
		$sql = "delete from permitdl where limitdate<?";
		$result = $this->delete($sql, $limitDate);
		return $result;
	}
	    
    function countPermitDL($goodsId=""){
      $sql = "select count(*) from permitdl";
      if(strlen($goodsId) == 0){
        $cnt = $this->count($sql);
      }else{ 
        $sql .= " where item_number=?";
        $cnt = $this->count($sql, $goodsId);
      }
      
      return $cnt;
    }

    /**
     * ダウンロードを許可している人のリスト
     * @param type $offset
     * @param type $num
     * @param int $order 1
     * @param String $goodsId 
     * @return type
     */
	function selectPermitDL($offset, $num, $order=2, $goodsId=""){
      switch($order){
        case 1: // 登録日の新しいもの
          $orderString = "tradedate";
          break;
        case 2: //　登録の古いもの
          $orderString = "tradedate desc";
          break;
        default: // 登録日の新しいもの
          $orderString = "tradedate";
          break;
      }      
      if(strlen($goodsId) == 0){
		$sql = "select txn_id,permitdl.item_number, payer_id, payer_email, permitdl.downloadcnt, tradedate, limitdate, item_name from permitdl left join goods on permitdl.item_number=goods.item_number order by "
                . $orderString . " limit " . $offset . "," . $num;
		$rows = $this->select($sql);
      }else{
		$sql = "select txn_id,permitdl.item_number, payer_id, payer_email, permitdl.downloadcnt, tradedate, limitdate, item_name from permitdl left join goods on permitdl.item_number=goods.item_number where permitdl.item_number=? order by "
                . $orderString . " limit " . $offset . "," . $num;
		$rows = $this->select($sql, $goodsId);        
      }
      return $rows;
	}

    function getPermitDL($tradeId){
      $sql = "select txn_id,permitdl.item_number, payer_id, payer_email, permitdl.downloadcnt, tradedate, limitdate, item_name,price,mc_currency from permitdl left join goods on permitdl.item_number=goods.item_number where txn_id=?";
      $ret = $this->get($sql, $tradeId);
      return $ret;
    }

	function isExistPermitDL($txn_id){
		$sql = "select txn_id from permitdl where txn_id=?";
		$ret = $this->isExist($sql, $txn_id);
		return $ret;
	}    
    
  function __construct() {
    try {
      if(defined("_PHP_PDO_MY_CNF_")){
      // PHP 5.3.6より前で、DB内の文字化けが気になる場合
      $this->dbh = new PDO('mysql:host=localhost;dbname=' . $this->dbName,
                      $this->userName, $this->userPassword, array(
                          PDO::MYSQL_ATTR_READ_DEFAULT_FILE => _PHP_PDO_MY_CNF_
                      ));
      }else if(defined("_PHP_PDO_COMMAND_")){
        $this->dbh = new PDO('mysql:host=localhost;dbname=' . $this->dbName,
                      $this->userName, $this->userPassword, array(
                          PDO::MYSQL_ATTR_INIT_COMMAND => _PHP_PDO_COMMAND_
                      ));
        
      }else if(defined ("_PHP_PDO_CHARSET_")){
        $this->dbh = new PDO('mysql:host=localhost;dbname=' . $this->dbName . ";charset="
                . _PHP_PDO_CHARSET_,
                      $this->userName, $this->userPassword);
      }else{
        $this->dbh = new PDO('mysql:host=localhost;dbname=' . $this->dbName . ";charset=utf8",
                      $this->userName, $this->userPassword);
      }
    } catch (PDOException $e) {
      throw new Exception('Error Not Open DB');
    }
  }

  function __destruct() {
    if (isset($this->dbh)) {
      $this->dbh = NULL;
    }
  }

  /**
   * エラーログを記録する
   *
   * @param String $logData
   * @return bool
   */
  function insertErrLog($logData) {
    $sql = "insert into tblerrorlog values(?,null)";
    $this->insert($sql, $logData);
  }

    function countErrLog(){
      $sql = "select count(*) from tblerrorlog";
      return $this->count($sql);
    }

	function selectErrLog($offset, $num, $order=2){
      switch($order){
        case 1: // 登録日の新しいもの
          $orderString = "logdate";
          break;
        case 2: //　登録の古いもの
          $orderString = "logdate desc";
          break;
        default: // 登録日の新しいもの
          $orderString = "logdate";
          break;
      }      
      $sql = "select logdata,logdate from tblerrorlog order by "
                . $orderString . " limit " . $offset . "," . $num;
      $rows = $this->select($sql);
      return $rows;
	}

	function deleteErrLogOld($term){
		$sql = "delete from tblerrorlog where logdate<subdate(curdate(), interval ? month)";
		$result = $this->delete($sql, $term);
		return $result;
	}
    
	/**
	 * ログを記録する
	 *
	 * @param String $logData
	 * @return bool
	 */
	function insertLog($logData){
		$sql = "insert into tbllog values(?,null)";
		$this->insert($sql, $logData);
	}
	
    function countLog(){
      $sql = "select count(*) from tbllog";
      return $this->count($sql);
    }

	function selectLog($offset, $num, $order=2){
      switch($order){
        case 1: // 登録日の新しいもの
          $orderString = "logdate";
          break;
        case 2: //　登録の古いもの
          $orderString = "logdate desc";
          break;
        default: // 登録日の新しいもの
          $orderString = "logdate";
          break;
      }      
      $sql = "select logdata,logdate from tbllog order by "
                . $orderString . " limit " . $offset . "," . $num;
      $rows = $this->select($sql);
      return $rows;
	}

	function deleteLogOld($term){
		$sql = "delete from tbllog where logdate<subdate(curdate(), interval ? month)";
		$result = $this->delete($sql, $term);
		return $result;
	}
    
	/**
	 * 取引を記録する
	 *
	 * @param String $txn_id
     * @param String $item_number
     * @param String $receiver_id
	 * @param String $receiver_email
	 * @param String $payer_id
	 * @param String $payer_email
	 * @param String $payment_date
     * @param String $mc_gross
     * @param String $mc_currency 
	 * @return bool
	 */
	function insertTrade($txn_id, $item_number, $receiver_id, $receiver_email, $payer_id, $payer_email, $payment_date, $mc_gross, $mc_currency){
		$sql = "insert into trade(txn_id, item_number, receiver_id, receiver_email, payer_id, payer_email, payment_date, mc_gross, mc_currency) values(?,?,?,?,?,?,?,?,?)";
		$result = $this->insert($sql, $txn_id, $item_number, $receiver_id, $receiver_email, $payer_id, $payer_email, $payment_date, $mc_gross, $mc_currency);
		return $result;
	}
	
	/**
	 * 取引が記録されているか
	 *
	 * @param string $txn_id
	 * @return bool
	 */
	function isExistTrade($txn_id){
		$sql = "select txn_id from trade where txn_id=?";
		$ret = $this->isExist($sql, $txn_id);
		return $ret;
	}

	function deleteTradeOld($term){
		$sql = "delete from trade where regdate<subdate(curdate(), interval ? month)";
		$result = $this->delete($sql, $term);
		return $result;
	}
    
	/**
	 * ダウンロード許可が与えられている場合、商品番号を得る
	 *
	 * @param string $txn_id
	 * @return string
	 */
	function getItemNumber($txn_id){
		$sql = "select item_number,downloadcnt,limitdate from permitdl where (txn_id=?)";
		$ret = $this->get($sql, $txn_id);
		return $ret;
	}
	
  /**
   * サイトの管理用データ
   * @param <type> $siteKey
   * @param <type> $siteData 
   */
  function insertSite($siteKey, $siteData) {
    $sql = "insert into tblsite(sitekey,sitedata) values(?,?)";
    $this->insert($sql, $siteKey, $siteData);
  }

  function deleteSite($siteKey) {
    $sql = "delete from tblsite where sitekey=?";
    $this->delete($sql, $siteKey);
  }

  function selectSite($siteKey) {
    $sql = "select sitekey,sitedata from tblsite where sitekey=?";
    $ret = $this->select($sql, $siteKey);
    return $ret;
  }

  /**
   * サイトのキーに対応する値を返す
   * @param type $siteKey
   * @return type
   */
  function getSiteData($siteKey) {
    $sql = "select sitedata from tblsite where sitekey=?";
    $ret = $this->get($sql, $siteKey);
    if(is_null($ret)){
      return $ret;
    }else{
      return $ret['sitedata'];
    }
  }
  
  /**
   * select
   * 　第一引数は、SQL文。その後は、引数。
   *
   * @return array
   */
  protected function select() {
    try {
      if (func_num_args() == 0) {
        throw new Exception('Error DB Select');
      }
      $stmt = $this->dbh->prepare(func_get_arg(0));
      $param = array_slice(func_get_args(), 1);
      if ($stmt->execute($param)) {
        return $stmt->fetchAll();
      }
      throw new Exception('Error DB Select');
    } catch (Exception $e) {
      throw new Exception('Error DB Select');
    }
  }

  /**
   * 検索結果があるかを返す
   * 第一引数は、SQL文。その後は、引数。
   *
   * @return bool
   */
  protected function isExist() {
    try {
      if (func_num_args() == 0) {
        throw new Exception('Error DB isExist');
      }
      $stmt = $this->dbh->prepare(func_get_arg(0));
      $param = array_slice(func_get_args(), 1);
      if ($stmt->execute($param)) {
        if ($stmt->fetch() === false) {
          return false;
        } else {
          return true;
        }
      }
      throw new Exception('Error DB isExist');
    } catch (Exception $e) {
      throw new Exception('Error DB isExist');
    }
  }

  /**
   * 検索結果の１つのみを返す。無い場合、NULL
   *
   * @return array
   */
  protected function get() {
    try {
      if (func_num_args() == 0) {
        throw new Exception('Error DB get');
      }
      $stmt = $this->dbh->prepare(func_get_arg(0));
      $param = array_slice(func_get_args(), 1);
      if ($stmt->execute($param)) {
        $ret = $stmt->fetchAll();
        return $ret[0];
      }
      throw new Exception('Error DB get');
    } catch (Exception $e) {
      throw new Exception('Error DB get');
    }
  }

  protected function count() {
    try {
      if (func_num_args() == 0) {
        throw new Exception('Error DB get');
      }
      $stmt = $this->dbh->prepare(func_get_arg(0));
      $param = array_slice(func_get_args(), 1);
      if ($stmt->execute($param)) {
        $ret = $stmt->fetchAll();
        return $ret[0][0];
      }
      throw new Exception('Error DB get');
    } catch (Exception $e) {
      throw new Exception('Error DB get');
    }
  }

  /**
   * insert
   * 　第一引数は、SQL文。その後は、引数。
   *
   * @return void
   */
  protected function insert() {
    try {
      if (func_num_args() == 0) {
        throw new Exception('Error DB insert');
      }
      $stmt = $this->dbh->prepare(func_get_arg(0));
      $param = array_slice(func_get_args(), 1);
      if ($stmt->execute($param)) {
        return $stmt->rowCount();
      }
      throw new Exception('Error DB insert');
    } catch (Exception $e) {
      throw new Exception('Error DB insert');
    }
  }
	
  /**
   * update
   * 　第一引数は、SQL文。その後は、引数。
   *
   * @return integer
   */
  protected function update() {
    try {
      if (func_num_args() == 0) {
        throw new Exception('Error DB update');
      }
      $stmt = $this->dbh->prepare(func_get_arg(0));
      $param = array_slice(func_get_args(), 1);
      if ($stmt->execute($param)) {
        return $stmt->rowCount();
      }
      throw new Exception('Error DB update');
    } catch (Exception $e) {
      throw new Exception('Error DB update');
    }
  }

  /**
   * delete
   * 　第一引数は、SQL文。その後は、引数。
   *
   * @return integer
   */
  protected function delete() {
    try {
      if (func_num_args() == 0) {
        throw new Exception('Error DB delete');
      }
      $stmt = $this->dbh->prepare(func_get_arg(0));
      $param = array_slice(func_get_args(), 1);
      if ($stmt->execute($param)) {
        return $stmt->rowCount();
      }
      throw new Exception('Error DB delete');
    } catch (Exception $e) {
      throw new Exception('Error DB delete');
    }
  }
}

?>