<?php
/**
 * 基本ライブラリ HTML関連
 * 
 * @package basic
 * @modulegroup lib
 * @create 2009/08/18
 */

/**
 * 文字列をHTML表示可能な文字列に変換する
 *
 * @param unknown_type $str
 */
function string2html($str){
	return htmlspecialchars($str, ENT_QUOTES, "UTF-8");
}

/**
 * メールアドレスの文字列の正当性を調べる
 *
 * @param string $email
 * @return mixed 正常:true エラーの場合 -1:短い -2:長い -3:正しくない
 */
function validateEmailString($email){
	if(strlen($email) == 0){
		return -1;
	}else if(strlen($email) > 255){
		return -2;	
	}else if(!preg_match("/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+([\.][a-z0-9-]+)+$/i",$email)){
		// 不正な文字が含まれる場合
		return -3;
	}else{
		return true;
	}
}

/**
 * パスワードの文字列の妥当性を調べる
 * @param type $password
 * @param type $min
 * @param type $max
 * @return boolean
 */
function validatePasswordString($password, $min, $max){
	if(strlen($password) < $min){
		return -1;
	}else if(strlen($password) > $max){
		return -2;	
	}else if(!preg_match("/^[A-Za-z0-9]+$/i",$password)){
		// 不正な文字が含まれる場合
		return -3;
	}else{
		return true;
	}  
}

/**
 * 改行をCRに統一する
 *
 * @param unknown_type $str
 * @return unknown
 */
function nl2cr($str){
	$str1 = mb_ereg_replace("\r\n", "\n", $str);
	$str2 = mb_ereg_replace("\r", "\n", $str1);
	return $str2;
}

?>